<?php

namespace Picea\Ui\Form;

use Picea\Extension\Extension;
use Picea\Extension\ExtensionTrait;
use Picea\Ui\Common\UiElement;
use Picea\Ui\Component\UiMessage;
use Picea\Ui\Component\UiPopup;

class Ui extends UiElement implements Extension {
    use ExtensionTrait;

    public string $token = "ui";

    protected /* ? mixed */ $value;

    protected string $name;

    public function parse(\Picea\Compiler\Context &$context, ?string $arguments, string $token, array $options = []) : string
    {
        switch(true) {
            case in_array('checkbox', $options):
                $input = new UiCheckbox();
                break;

            case in_array('color', $options):
                $input = new UiColor();
                break;

            case in_array('date', $options):
                $input = new UiDate();
                break;

            case in_array('datetime', $options):
                $input = new UiDatetime();
                break;

            case in_array('email', $options):
                $input = new UiEmail();
                break;

            case in_array('file', $options):
                $input = new UiFile();
                break;

            case in_array('form', $options):
            case in_array('endform', $options):
                $input = new UiForm();
                break;

            case in_array('hidden', $options):
                $input = new UiHidden();
                break;

            case in_array('img', $options):
            case in_array('image', $options):
                $input = new UiImage();
                break;

            case in_array('input', $options):
                $input = new UiInput();
                break;

            case in_array('numeric', $options):
                $input = new UiNumeric();
                break;

            case in_array('password', $options):
                $input = new UiPassword();
                break;

            case in_array('radio', $options):
                $input = new UiRadio();
                break;

            case in_array('range', $options):
                $input = new UiRange();
                break;

            case in_array('search', $options):
                $input = new UiSearch();
                break;

            case in_array('select', $options):
                $input = new UiSelect();
                break;

            case in_array('tel', $options):
                $input = new UiTel();
                break;

            case in_array('text', $options):
                $input = new UiText();
                break;

            case in_array('textarea', $options):
                $input = new UiTextarea();
                break;

            case in_array('time', $options):
                $input = new UiText();
                break;

            case in_array('url', $options):
                $input = new UiUrl();
                break;

            case in_array('week', $options):
                $input = new UiWeek();
                break;

            case in_array('popup', $options):
                $input = new UiPopup();
                break;

            case in_array('message', $options):
                $input = new UiMessage();
                break;
        }

        if (empty($input)) {
            throw new \Exception("Missing token ? $token $arguments");
        }

        return $input->parse($context, $arguments, $token, $options);
    }
}
